import sys

sys.path.append('..')
from common.core import *
from common.audio import *
from common.writer import *
from common.gfxutil import topleft_label

# This purpose of these exercises is to make sure audio works on your computer
# and to show you a useful debugging technique for the audio we'll be working
# with using Audacity

# Excercise 1: Run this code as is and press some of the buttons defined in
# MainWidget.  Make sure you hear the sounds you expect to hear

# Excercise 2: Edit the left and right keycode so instead of just adding or
# subtracting a fixed number to the frequency, hitting left and right goes up
# and down 1 halfstep (or pick a different interval)

# Excercise 3: Pressing 'r' while running the code starts a sound file recording, and pressing it again stops the recording.
# Edit self.info.text to say "recording" while it's recording and nothing when it isn't.
# While running the code, press the 'r' key to start recording a
# wave file.  Press some buttons to change the sound. Then press 'r' again to
# stop recording. When you close the application you will have a wave file
# containing the sound data. Open it with Audacity and look at different parts
# of the waveform. Can you see where you changed frequencies? Is it continuous
# or are there discontinuities? We'll be fixing this in this week's pset!

class SineGenerator(object):
    def __init__(self):
        super(SineGenerator, self).__init__()
        self.frame = 0
        self.freq = 0
        self.gain = 0.5

    def set_freq(self, f) :
        self.freq = f

    def get_freq(self) :
        return self.freq

    def set_gain(self, g) :
        self.gain = np.clip(g, 0, 1)

    def get_gain(self) :
        return self.gain

    def generate(self, num_frames, num_channels) :
        frames = np.arange(self.frame, self.frame + num_frames)
        phase = (2.0 * np.pi) * self.freq * frames / Audio.sample_rate
        output = self.gain * np.sin( phase, dtype = np.float64)
        self.frame += num_frames

        return (output, True)


class MainWidget(BaseWidget) :
    def __init__(self):
        super(MainWidget, self).__init__()

        self.writer = AudioWriter('data')
        self.audio = Audio(1, self.writer.add_audio)

        self.gen = SineGenerator()
        self.audio.set_generator(self.gen)

        self.info = topleft_label()
        self.add_widget(self.info)

    def on_update(self) :
        self.audio.on_update()
        self.info.text = 'load:%.2f\n' % self.audio.get_cpu_load()
        self.info.text += 'gain: %.2f\n' % self.gen.get_gain()
        self.info.text += 'freq: %.2f' % self.gen.get_freq()

    def on_key_down(self, keycode, modifiers):
        print 'key-down', keycode, modifiers
        if keycode[1] == '0':
            self.gen.set_freq(0)

        elif keycode[1] == '1':
            self.gen.set_freq(440)

        elif keycode[1] == '2':
            self.gen.set_freq(880)

        elif keycode[1] == 'up':
            self.gen.set_gain( self.gen.get_gain() + 0.05 )

        elif keycode[1] == 'down':
            self.gen.set_gain( self.gen.get_gain() - 0.05 )

        elif keycode[1] == 'left':
            self.gen.set_freq( self.gen.get_freq() - 100 )

        elif keycode[1] == 'right':
            self.gen.set_freq( self.gen.get_freq() + 100 )

        elif keycode[1] == 'r':
            self.writer.toggle()


run(MainWidget)