# lecture1a.py - basic sine synthesis


# set up pyAudio for streaming. import pyaudio module, create the audio interface and
# create an output stream with the basic parameters:
# channel = 1
# sample rate = 44100
# audio format is 32-bit floating point

import pyaudio
import time

def main(get_data):
    print 'Hello Audio!'
    audio = pyaudio.PyAudio()
    stream = audio.open(format = pyaudio.paFloat32,
                         channels = 1,
                         rate = 44100,
                         frames_per_buffer = 256,
                         output_device_index = None,
                         output = True,
                         input = False)

    while 1:
        frames = stream.get_write_available()
        print frames
        if frames > 0:
            data = get_data(frames);
            stream.write(data.astype(np.float32).tostring())

        time.sleep(0.01)



# now, let's make some noise. Literally. In the callback, we need to return
# the number of samples requested. So for starters, let's just return some
# random values, which should sound like white-noise or static. import random
# for that.

import random
import numpy as np

def make_noise(num_frames) :
    data = np.zeros(num_frames)
    f = 0
    while f < num_frames:
        data[f] = 0.1 * random.random()
        f += 1
    return data

# main(make_noise)


# let's make a tone out of a pure sine wave. So for that we need the sin
# function. import math sin(a) is very high frequency. Let's chop down the
# frequency by 10. ok. more in normal hearing range, but it is clicky. Why
# is that?
from math import sin

def make_some_sine(num_frames) :
    data = np.zeros(num_frames)
    f = 0
    while f < num_frames:
        data[f] = 0.1 * sin(f * 0.1)
        f += 1
    return data

# main(make_some_sine)


# We need a global sense of the frame so that the sin function gets
# a contiuous and smooth input value. Instead of using the range value which
# always starts at 0, let's use the global frame value
frame = 0
def make_smooth_sine(num_frames) :
    global frame
    data = np.zeros(num_frames)
    f = 0
    while f < num_frames:
        data[f] = 0.1 * sin(frame * 0.1)
        f += 1
        frame += 1
    return data

# main(make_smooth_sine)


# now lets do a better job of picking the exact frequency we want.
# say we want to make a tone that is exactly A=440 (orchestra tuning pitch)
# so, we calculate the argument to sine by useing 2pi * f * N / sample_rate

freq = 440
sample_rate = 44100

def make_sine(frames):
    global frame
    values = []
    for i in range(frames):
        ph = 2 * np.pi * freq * frame / sample_rate
        val = 0.1 * ( sin(ph) )
        values.append(val)
        frame += 1
    return np.array(values, dtype=np.float32)

main(make_sine)



# effeciency is pretty important and the current audio loop is very
# ineffecient. Luckily, we have an awesome python lib called numpy that can
# do math / linear algebra / arrays / matricies very fast. import numpy as np
# and use np.arange() with np.sin.

def make_fast_sine(num_frames) :
    global frame

    frames = np.arange(frame, frame + num_frames)
    ph = 2.0 * np.pi * freq * frames / sample_rate
    output = .1 * np.sin( ph, dtype = np.float32)
    frame += num_frames

    return output

# main(make_fast_sine)


def test_speed():
    import timeit
    print 'slow', timeit.timeit("make_smooth_sine(512)",
       setup='from __main__ import make_smooth_sine', number=1000)

    print 'fast', timeit.timeit("make_fast_sine(512)",
       setup='from __main__ import make_fast_sine', number=1000)

# test_speed()
