# lecture1b.py

import sys

sys.path.append('..')
from common.core import *
from common.audio import *
from common.writer import *
from common.gfxutil import topleft_label


class SineGenerator(object):
    def __init__(self):
        super(SineGenerator, self).__init__()
        self.frame = 0
        self.freq = 0
        self.gain = 0.1


    def set_freq(self, f) :
        self.freq = f

    def get_freq(self) :
        return self.freq

    def set_gain(self, g) :
        self.gain = np.clip(g, 0, 1)

    def get_gain(self) :
        return self.gain

    def generate(self, num_frames, num_channels) :
        n = np.arange(self.frame, self.frame + num_frames)
        output = self.gain * np.sin( 2.0 * np.pi * self.freq * n / Audio.sample_rate )
        self.frame += num_frames

        return (output, True)


class MainWidget(BaseWidget) :
    def __init__(self):
        super(MainWidget, self).__init__()

        self.writer = AudioWriter('data')
        self.audio = Audio(1, self.writer.add_audio)

        self.gen = SineGenerator()
        self.audio.set_generator(self.gen)

        self.info = topleft_label()
        self.add_widget(self.info)

    def on_update(self) :
        self.audio.on_update()
        self.info.text = 'load:%.2f\n' % self.audio.get_cpu_load()
        self.info.text += 'gain: %.2f\n' % self.gen.get_gain()
        self.info.text += 'freq: %.2f' % self.gen.get_freq()

    def on_key_down(self, keycode, modifiers):
        print 'key-down', keycode, modifiers
        if keycode[1] == '0':
            self.gen.set_freq(0)

        elif keycode[1] == '1':
            self.gen.set_freq(440)

        elif keycode[1] == '2':
            self.gen.set_freq(880)

        elif keycode[1] == 'up':
            self.gen.set_gain( self.gen.get_gain() + 0.05 )

        elif keycode[1] == 'down':
            self.gen.set_gain( self.gen.get_gain() - 0.05 )

        elif keycode[1] == 'left':
            self.gen.set_freq( self.gen.get_freq() - 100 )

        elif keycode[1] == 'right':
            self.gen.set_freq( self.gen.get_freq() + 100 )

        elif keycode[1] == 'r':
            self.writer.toggle()


run(MainWidget)
